unit UnitStatusBar;

{   GsvStatusBar
     -   , 
     .

      :
  http://www.tomsk.net/2q/gurin/delphicompstatusbar.html

   . , 
  mailto:gurin@mail.tomsknet.ru
  http://www.tomsk.net/2q/gurin/index.html

}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls, ToolWin, ComCtrls, Menus, ActnList, ImgList, Buttons, StdCtrls,
  GsvStatusBar;

const
  //   
  errNeutral  = -1;
  errSuccess  = 0;
  errWait     = 1;
  errHint     = 2;
  errWarning  = 3;
  errError    = 4;

type
  TFormStatusBar = class(TForm)
    ImageList:          TImageList;
    imgStatusBar:       TImageList;
    ActionList:         TActionList;

    actStatusClear:     TAction;
    actStatusSuccess:   TAction;
    actStatusHint:      TAction;
    actStatusWarning:   TAction;
    actStatusError:     TAction;
    actStatusOperation: TAction;
    actStatusCancel:    TAction;
    actStatusHelp:      TAction;
    actStatusExit:      TAction;

    MainMenu:           TMainMenu;
    mnuStatus:          TMenuItem;
    mnuStatusClear:     TMenuItem;
    mnuStatusSuccess:   TMenuItem;
    mnuStatusHint:      TMenuItem;
    mnuStatusWarning:   TMenuItem;
    mnuStatusError:     TMenuItem;
    mnuStatusSep1:      TMenuItem;
    mnuStatusOperation: TMenuItem;
    mnuStatusCancel:    TMenuItem;
    mnuStatusSep2:      TMenuItem;
    mnuStatusHelp:      TMenuItem;
    mnuStatusSep3:      TMenuItem;
    mnuStatusExit:      TMenuItem;

    ControlBar:         TControlBar;
    ToolBarStatus:      TToolBar;
    btnStatusClear:     TToolButton;
    btnStatusSuccess:   TToolButton;
    btnStatusHint:      TToolButton;
    btnStatusWarning:   TToolButton;
    btnStatusError:     TToolButton;
    btnStatusSep1:      TToolButton;
    btnStatusOperation: TToolButton;
    btnStatusCancel:    TToolButton;
    btnStatusSep2:      TToolButton;
    btnStatusHelp:      TToolButton;
    btnStatusSep3:      TToolButton;
    btnStatusExit:      TToolButton;

    procedure OnStatusDraw(aImageIndex: Integer; const aText: String);
    procedure OnStatusClick(Sender: TObject);

    procedure FormCreate(Sender: TObject);

    procedure actStatusClearExecute(Sender: TObject);
    procedure actStatusSuccessExecute(Sender: TObject);
    procedure actStatusHintExecute(Sender: TObject);
    procedure actStatusWarningExecute(Sender: TObject);
    procedure actStatusErrorExecute(Sender: TObject);
    procedure actStatusOperationExecute(Sender: TObject);
    procedure actStatusCancelExecute(Sender: TObject);
    procedure actStatusHelpExecute(Sender: TObject);
    procedure actStatusExitExecute(Sender: TObject);

  private
    //{
    Status:  TGsvStatusBar;
    //}
    CancelProcess: Boolean;

    function  HelpIndex: Integer;
    procedure BeginWaiting;
    procedure EndWaiting;
  end;

var
  FormStatusBar: TFormStatusBar;

implementation

{$R *.DFM}

procedure TFormStatusBar.FormCreate(Sender: TObject);
begin
  //{
  Status                 := TGsvStatusBar.Create(self);
  Status.Parent          := Self;
  Status.Images          := imgStatusBar;
  Status.OnDraw          := OnStatusDraw;
  Status.OnClick         := OnStatusClick;
  Status.ImageHelpIndex  := 5;
  Status.BevelInner      := bvLowered;
  Status.BevelOuter      := bvNone;
  Status.Hint            := '     ';
  //}
  Application.HelpFile   := ChangeFileExt(Application.ExeName, '.hlp');
end;

procedure TFormStatusBar.OnStatusDraw(aImageIndex: Integer;
  const aText: String);
begin
  if aImageIndex = errError then
    MessageBeep(0);
end;

procedure TFormStatusBar.OnStatusClick(Sender: TObject);
begin
  if Status.HelpIndex > 0 then
    Application.HelpContext(Status.HelpIndex);
end;

procedure TFormStatusBar.actStatusClearExecute(Sender: TObject);
begin
  Status.Clear;
end;

procedure TFormStatusBar.actStatusSuccessExecute(Sender: TObject);
begin
  Status.Draw(errSuccess, '   ', HelpIndex);
end;

procedure TFormStatusBar.actStatusHintExecute(Sender: TObject);
begin
  Status.Draw(errHint, '   ', HelpIndex);
end;

procedure TFormStatusBar.actStatusWarningExecute(Sender: TObject);
begin
  Status.Draw(errWarning, ' - ', HelpIndex);
end;

procedure TFormStatusBar.actStatusErrorExecute(Sender: TObject);
begin
  Status.Draw(errError, '  ', HelpIndex);
end;

procedure TFormStatusBar.actStatusOperationExecute(Sender: TObject);
var
  i: Integer;
begin
  Status.Draw(errWait, ' ');
  BeginWaiting;
  CancelProcess := False;
  for i := 0 to 100 do begin
    Status.Caption := ' ' + IntToStr(i) + '%';
    Sleep(100);
    Application.ProcessMessages;
    if CancelProcess then
      break;
  end;
  EndWaiting;
  if CancelProcess then
    Status.Draw(errWarning, ' ')
  else
    Status.Draw(errSuccess, ' ');
end;

procedure TFormStatusBar.actStatusCancelExecute(Sender: TObject);
begin
  CancelProcess := True;
end;

procedure TFormStatusBar.actStatusHelpExecute(Sender: TObject);
begin
  actStatusHelp.Checked := not actStatusHelp.Checked;
end;

procedure TFormStatusBar.actStatusExitExecute(Sender: TObject);
begin
  Close;
end;

function TFormStatusBar.HelpIndex: Integer;
begin
  if actStatusHelp.Checked then
    result := 10
  else
    result := 0;
end;

procedure TFormStatusBar.BeginWaiting;
begin
  actStatusClear.Enabled     := False;
  actStatusSuccess.Enabled   := False;
  actStatusHint.Enabled      := False;
  actStatusWarning.Enabled   := False;
  actStatusError.Enabled     := False;
  actStatusOperation.Enabled := False;
  actStatusCancel.Enabled    := True;
  actStatusHelp.Enabled      := False;
  actStatusExit.Enabled      := False;
end;

procedure TFormStatusBar.EndWaiting;
begin
  actStatusClear.Enabled     := True;
  actStatusSuccess.Enabled   := True;
  actStatusHint.Enabled      := True;
  actStatusWarning.Enabled   := True;
  actStatusError.Enabled     := True;
  actStatusOperation.Enabled := True;
  actStatusCancel.Enabled    := False;
  actStatusHelp.Enabled      := True;
  actStatusExit.Enabled      := True;
end;

end.
