unit GsvStatusBar;

{   -  ,   
   (  ).   
     ,   .
          
   .

     :
  http://www.tomsk.net/2q/gurin/delphicompstatusbar.html

   . , 
  mailto:gurin@mail.tomsknet.ru
  http://www.tomsk.net/2q/gurin/index.html

  :
    Clear
        -  Draw(-1, '', 0)
    Draw(aImageIndex: Integer; const aText: String; aHelpIndex: Integer);
        ,  , 
          ,   ,
          
    Draw(aImageIndex: Integer; const aText: String);
       Draw(aImageIndex, aText, 0);
    IndexDraw(aIndex: Integer; const aArgument: String);
        OnIndexError(aIndex, aArgument)
    IndexDraw(aIndex: Integer);
        OnIndexError(aIndex, '')

  :
    OnIndexDraw(aIndex: Integer; const aArgument: String);
              
               Draw.
            .
    OnClick(Sender: TObject);
            , 
             
    OnDraw(aImageIndex: Integer; const aText: String);
         Draw.  
            
         

  Published :
    Images: TCustomImageList;
         ImageList
    HelpImageIndex: Integer;
        Images    
    Caption: String;
               
  Public :
    ImageIndex: Integer;
             
    HelpIndex: Integer;
           
   ShowHint     
   HelpIndex:
    ShowHint := HelpIndex > 0;
}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ComCtrls, ExtCtrls, ActnList, StdCtrls, ImgList;

type
  TGsvStatusBarIndexDrawEvent = procedure(aIndex: Integer;
                                const aArgument: String) of object;
  TGsvStatusBarDrawEvent      = procedure(aImageIndex: Integer;
                                const aText: String) of object;

  TGsvStatusBar = class(TCustomPanel)
  private
    FImages:          TCustomImageList;
    FImageChangeLink: TChangeLink;
    FImageHelpIndex:  Integer;
    FImageIndex:      Integer;
    FHelpIndex:       Integer;
    FImageWidth:      Integer;

    FOnIndexDraw:     TGsvStatusBarIndexDrawEvent;
    FOnDraw:          TGsvStatusBarDrawEvent;

    procedure   SetImages(const Value: TCustomImageList);
    procedure   SetImageIndex(aImageIndex: Integer);
    procedure   SetHelpIndex(aHelpIndex: Integer);

  protected
    function    CalcImagePoint(aWidth, aHeight: Integer): TPoint;
    procedure   PaintImage; virtual;
    procedure   PaintHelpImage; virtual;
    procedure   Paint; override;
    procedure   Notification(aComponent: TComponent;
                aOperation: TOperation); override;

  public
    constructor Create(Owner: TComponent); override;
    destructor  Destroy; override;

    //  
    procedure   Clear;
    procedure   Draw(aImageIndex: Integer;
                const aText: String; aHelpIndex: Integer); overload;
    procedure   Draw(aImageIndex: Integer; const aText: String); overload;
    procedure   IndexDraw(aIndex: Integer; const aArgument: String); overload;
    procedure   IndexDraw(aIndex: Integer); overload;

    property    ImageIndex: Integer read FImageIndex write SetImageIndex;
    property    HelpIndex: Integer read FHelpIndex write SetHelpIndex;

  published
    property    Align;
    property    Anchors;
    property    BevelInner;
    property    BevelOuter;
    property    BevelWidth;
    property    BorderWidth;
    property    BorderStyle;
    property    Caption;
    property    Color;
    property    Ctl3D;
    property    Enabled;
    property    Font;
    property    Locked;
    property    ParentColor;
    property    ParentCtl3D;
    property    ParentFont;
    property    PopupMenu;
    property    Visible;
    property    OnContextPopup;
    property    OnClick;
    property    Images: TCustomImageList read FImages write SetImages;
    property    ImageHelpIndex: Integer
                read FImageHelpIndex write FImageHelpIndex;
    property    OnIndexDraw: TGsvStatusBarIndexDrawEvent
                read FOnIndexDraw write FOnIndexDraw;
    property    OnDraw: TGsvStatusBarDrawEvent
                read FOnDraw write FOnDraw;
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('Gsv', [TGsvStatusBar]);
end;

// Private

procedure TGsvStatusBar.SetImages(const Value: TCustomImageList);
begin
  if FImages <> Value then begin
    if FImages <> nil then
      FImages.UnRegisterChanges(FImageChangeLink);
    FImages := Value;
    if FImages <> nil then begin
      FImages.RegisterChanges(FImageChangeLink);
      FImages.FreeNotification(Self);
    end;
  end;  
end;

procedure TGsvStatusBar.SetImageIndex(aImageIndex: Integer);
begin
  FImageIndex := aImageIndex;
  Refresh;
end;

procedure TGsvStatusBar.SetHelpIndex(aHelpIndex: Integer);
begin
  FHelpIndex := aHelpIndex;
  ShowHint   := (FHelpIndex > 0);
  Refresh;
end;

// Protected

function TGsvStatusBar.CalcImagePoint(aWidth, aHeight: Integer): TPoint;
var
  r:    TRect;
  h, g: Integer;
begin
  r := GetClientRect;
  h := r.Bottom - r.Top - BevelWidth * 2;
  g := (h - aHeight) div 2;
  if g < 0 then
    g := 0;
  result.X := r.Right - BevelWidth - aWidth;
  result.Y := r.Top + BevelWidth + g;
end;

procedure TGsvStatusBar.PaintImage;
var
  p: TPoint;
begin
  FImageWidth := 0;
  if Assigned(FImages) then begin
    if (FImageIndex >= 0) and (FImageIndex < FImages.Count) then begin
      p := CalcImagePoint(FImages.Width, FImages.Height);
      FImages.Draw(Canvas, p.X, p.Y, FImageIndex);
      FImageWidth := FImages.Width;
    end;
  end;
end;

procedure TGsvStatusBar.PaintHelpImage;
var
  p: TPoint;
begin
  if Assigned(FImages) then begin
    if (FImageHelpIndex >= 0) and (FImageHelpIndex < FImages.Count) then begin
      p := CalcImagePoint(FImages.Width, FImages.Height);
      FImages.Draw(Canvas, p.X - FImageWidth, p.Y, FImageHelpIndex);
    end;
  end;
end;

procedure TGsvStatusBar.Paint;
begin
  inherited;
  PaintImage;
  if FHelpIndex <> 0 then
    PaintHelpImage;
end;

procedure TGsvStatusBar.Notification(aComponent: TComponent;
  aOperation: TOperation);
begin
  inherited Notification(aComponent, aOperation);
  if (aOperation = opRemove) and (aComponent = FImages) then
    Images := nil;
end;

// Public *******************************

constructor TGsvStatusBar.Create(Owner: TComponent);
begin
  inherited Create(Owner);
  Height           := 18;
  Align            := alBottom;
  Alignment        := taLeftJustify;
  Caption          := '';
  DoubleBuffered   := True;
  FImageIndex      := -1;
  FImageHelpIndex  := -1;
  FImageChangeLink := TChangeLink.Create;
end;

destructor TGsvStatusBar.Destroy;
begin
  FImageChangeLink.Free;
  inherited;
end;

//  

procedure TGsvStatusBar.Clear;
begin
  Draw(-1, '', 0);
end;

procedure TGsvStatusBar.Draw(aImageIndex: Integer;
  const aText: String; aHelpIndex: Integer);
begin
  FImageIndex := aImageIndex;
  FHelpIndex  := aHelpIndex;
  Caption     := aText;
  ShowHint    := (FHelpIndex > 0);
  if Assigned(FOnDraw) then
    FOnDraw(aImageIndex, aText);
  Refresh;
end;

procedure TGsvStatusBar.Draw(aImageIndex: Integer; const aText: String);
begin
  Draw(aImageIndex, aText, 0);
end;

procedure TGsvStatusBar.IndexDraw(aIndex: Integer; const aArgument: String);
begin
  if Assigned(FOnIndexDraw) then
    FOnIndexDraw(aIndex, aArgument);
end;

procedure TGsvStatusBar.IndexDraw(aIndex: Integer);
begin
  if Assigned(FOnIndexDraw) then
    FOnIndexDraw(aIndex, '');
end;

end.

